/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: CatalinaBaseAction.java,v 1.7 2004/03/19 14:31:50 sauthieg Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin.catalina;

import javax.management.ObjectName;
import javax.servlet.http.HttpServletRequest;

import org.objectweb.jonas.jmx.J2eeObjectName;
import org.objectweb.jonas.webapp.jonasadmin.JonasTreeBuilder;
import org.objectweb.jonas.webapp.jonasadmin.WhereAreYou;
import org.objectweb.jonas.webapp.jonasadmin.deploy.BaseDeployAction;
import org.objectweb.jonas.webapp.taglib.TreeControl;
import org.objectweb.jonas.webapp.taglib.TreeControlNode;

/**
 * @author Michel-Ange ANTON
 */
abstract public class CatalinaBaseAction extends BaseDeployAction {

// --------------------------------------------------------- Public Methods
    /*
        protected CatalinaForm getCatalinaForm()
            throws Exception {
            // Form used
            CatalinaForm oForm = (CatalinaForm) m_Session.getAttribute("catalinaForm");
            if (oForm == null) {
                // Create form
                oForm = new CatalinaForm();
                m_Session.setAttribute("catalinaForm", oForm);
                // Object name used
                ObjectName oObjectName;
                // Get the service name (only one)
                oForm.setServiceName(JonasAdminJmx.extractValueMbeanName("name"
                    , JonasAdminJmx.getFirstMBeanName(CatalinaObjectName.catalinaServices())));
                // Object name Engine
                oObjectName = CatalinaObjectName.catalinaEngine(oForm.getServiceName());
                oForm.setDefaultHostName(getStringAttribute(oObjectName, "defaultHost"));
                // Object name Host
                oObjectName = CatalinaObjectName.catalinaHost(oForm.getServiceName()
                    , oForm.getDefaultHostName());
                oForm.setHostName(getStringAttribute(oObjectName, "name"));
            }
            return oForm;
        }
     */
    /**
     * Refresh the tree.
     *
     * @param p_Request Current instance
     * @throws Exception
     */
    protected void refreshTree(HttpServletRequest p_Request)
        throws Exception {
        // Refresh Security Tree
        refreshConnectorsTree(p_Request);
        // Refresh MBeans Tree
        refreshMBeansTree();
        // Force display to refresh
        m_WhereAreYou.setTreeToRefresh(true);
    }

    /**
     * Refresh the connectors branch tree.
     *
     * @param p_Request Current instance
     * @throws Exception
     */
    protected void refreshConnectorsTree(HttpServletRequest p_Request)
        throws Exception {
        // Get the server name
        ObjectName oObjectName = J2eeObjectName.J2EEServer(m_WhereAreYou.getCurrentDomainName()
            , m_WhereAreYou.getCurrentJonasServerName());
        String sNameServer = getStringAttribute(oObjectName, "ServerName");
        // Get the service name
        String sCurrentNodeNameItem = sNameServer + WhereAreYou.NODE_SEPARATOR + "protocols"
            + WhereAreYou.NODE_SEPARATOR + "connectors";
        // Get current tree
        TreeControl oControl = m_WhereAreYou.getTreeControl();
        // Get branch root node
        TreeControlNode oBranchRootNode = oControl.findNode(sCurrentNodeNameItem);
        // Enable auto-refresh mode
        oControl.enableAutoRefresh();
        // Remove old children
        TreeControlNode[] aoNodes = oBranchRootNode.findChildren();
        for (int i = 0; i < aoNodes.length; i++) {
            aoNodes[i].remove();
        }
        // Build node for the Service
        JonasTreeBuilder oBuilder = new JonasTreeBuilder();
        oBuilder.getCatalinaDetailConnectors(oBranchRootNode, m_Resources, p_Request);
        // Disable auto-refresh mode
        oControl.disableAutoRefresh();
    }

}
