/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999-2004 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ServletEndpointsFactory.java,v 1.3 2004/09/10 11:30:16 sauthieg Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_lib.naming.factory;


//import java
import java.util.Enumeration;
import java.util.Hashtable;

import javax.naming.Context;
import javax.naming.Name;
import javax.naming.RefAddr;
import javax.naming.Reference;
import javax.naming.spi.ObjectFactory;

import org.objectweb.jonas.common.Log;
import org.objectweb.jonas.ws.ServletEndpoints;

import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;


/**
 * Factory for ServletEndpoints instance
 * @author Guillaume Sauthier
 */
public class ServletEndpointsFactory implements ObjectFactory {

    /**
     * The Java type for which this factory knows how to create objects.
     */
    protected static final String FACTORY_TYPE = "org.objectweb.jonas.ws.ServletEndpoints";

    /**
     * logger
     */
    private static Logger logger = Log.getLogger(Log.JONAS_WS_PREFIX);


    /**
     * Creates an org.objectweb.jonas.ws.ServletEndpoints object using the location or reference
     * information specified.
     * @param obj the possibly null object containing location or reference
     * information that can be used in creating an object.
     * @param name the name of this object relative to nameCtx, or null if no
     * name is specified.
     * @param nameCtx the context relative to which the name parameter is
     * specified, or null if name is relative to the default initial context.
     * @param environment the possibly null environment that is used in
     * creating the object.
     * @return a newly created org.objectweb.jonas.ws.ServletEndpoints object with the specific
     * configuration; null if an object cannot be created.
     * @throws Exception if this object factory encountered an exception
     * while attempting to create an object, and no other object factories
     * are to be tried.
     */
    public Object getObjectInstance(Object obj, Name name, Context nameCtx,
                                    Hashtable environment) throws Exception {

        //Get the reference
        Reference ref = (Reference) obj;

        //Get the class name
        String clname = ref.getClassName();

        //Check the class name
        if (!clname.equals(FACTORY_TYPE)) {
            throw new Exception("Can not create object : required type is '" + FACTORY_TYPE + "', but found type is '" + clname + "'.");
        }

        ServletEndpoints se = new ServletEndpoints();
        String portsString =  (String) ref.get("port-names").getContent();

        if (portsString != null) {
            String[] ports = portsString.split(",");
            for (int i = 0; i < ports.length; i++) {
                logger.log(BasicLevel.DEBUG, "port : " + ports[i]);
                RefAddr addr = ref.get("port-" + ports[i]);
                if (addr != null) {
                    se.addEndpoint(ports[i], (String) addr.getContent());
                } else {
                    logger.log(BasicLevel.WARN, "port : port-" + ports[i] + "not found in Reference");
                    for (Enumeration e = ref.getAll(); e.hasMoreElements();) {
                        RefAddr ra = (RefAddr) e.nextElement();
                        logger.log(BasicLevel.WARN, ra.getType() + ":" + ra.getContent());
                    }
                }
                
            }
        }

        return se;
    }

}
