/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999-2004 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JWSDLLocator.java,v 1.1 2004/09/09 16:12:36 sauthieg Exp $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas_ws.deployment.api;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

import javax.wsdl.xml.WSDLLocator;

import org.xml.sax.InputSource;

/**
 * Used to load WSDL files.
 * @author Guillaume Sauthier
 */
public class JWSDLLocator implements WSDLLocator {

    /**
     * WSDL base URI
     */
    private URL baseURI;

    /**
     * URI of the latest imported WSDL
     */
    private String latestImportURI = null;

    /**
     * @param baseURI URL pointing to WSDL inside a jar file (of the form : jar:file://xxx/something.jar!/META-INF/wsdl/definition.wsdl)
     */
    public JWSDLLocator(URL baseURI) {
        this.baseURI = baseURI;
    }

    /**
     * @see javax.wsdl.xml.WSDLLocator#getBaseInputSource()
     */
    public InputSource getBaseInputSource() {
        try {
            return new InputSource(baseURI.openStream());
        } catch (IOException ioe) {
            return null;
        }
    }

    /**
     * @see javax.wsdl.xml.WSDLLocator#getBaseURI()
     */
    public String getBaseURI() {
        return baseURI.toExternalForm();
    }

    /**
     * @see javax.wsdl.xml.WSDLLocator#getImportInputSource(java.lang.String,
     *      java.lang.String)
     */
    public InputSource getImportInputSource(String ctx, String imported) {
        /*
         * If imported is relative to the BaseURI: <ol> <il>True : create a new
         * URL an load its content </il> <il>False : use the imported as URL
         * </il> </ol>
         */
        try {
            if (ctx == null) {
                URL url = getURL(baseURI, imported);
                latestImportURI = url.toExternalForm();
                return new InputSource(url.openStream());
            } else {
                URL url = getURL(new URL(ctx), imported);
                latestImportURI = url.toExternalForm();
                return new InputSource(url.openStream());
            }
        } catch (IOException ioe) {
            return null;
        }
    }

    /**
     * @see javax.wsdl.xml.WSDLLocator#getLatestImportURI()
     */
    public String getLatestImportURI() {
        return latestImportURI;
    }

    /**
     * @param contextURL context URL
     * @param spec locationURI to load (may be relative to contextURL)
     * @return Returns the merged URL of the URL of the spec if spec is absolute.
     * @throws MalformedURLException if cannot create new URL(contextURL, spec)
     */
    public static URL getURL(URL contextURL, String spec) throws MalformedURLException {
        if (contextURL != null) {
            File tempFile = new File(spec);
            if (tempFile.isAbsolute()) {
                return tempFile.toURL();
            }
        }

        try {
            return new URL(contextURL, spec);
        } catch (MalformedURLException e) {
            if (contextURL == null) {
                return new File(spec).toURL();
            } else {
                throw e;
            }
        }
    }

}