/**
 * JOnAS : Java(TM) OpenSource Application Server
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial Developer : Guillaume Sauthier
 * --------------------------------------------------------------------------
 * $Id: ClientModifier.java,v 1.4 2004/07/01 14:55:17 sauthieg Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ws.wsgen.modifier;

import java.io.File;
import java.util.Iterator;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import org.objectweb.jonas_ws.deployment.api.ServiceRefDesc;
import org.objectweb.jonas_ws.wsgen.WsGenException;
import org.objectweb.jonas_ws.wsgen.archive.Archive;
import org.objectweb.jonas_ws.wsgen.archive.Client;
import org.objectweb.jonas_ws.wsgen.ddmodifier.WsClientDDModifier;
import org.objectweb.jonas_ws.wsgen.generator.Generator;
import org.objectweb.jonas_ws.wsgen.generator.GeneratorFactory;

import org.objectweb.util.monolog.api.BasicLevel;

/**
 * Modify a given Client.
 *
 * @author Guillaume Sauthier
 */
public class ClientModifier extends ArchiveModifier {

    /** client */
    private Client client;

    /**
     * Creates a new ClientModifier object.
     *
     * @param client Client Archive
     */
    public ClientModifier(Client client) {
        super(client);
        this.client = client;
    }

    /**
     * Modify the current archive and return a modified archive.
     *
     * @return a modified archive.
     *
     * @throws WsGenException When Client Generation or storing phase fails.
     */
    public Archive modify() throws WsGenException {

        getLogger().log(BasicLevel.INFO, "Processing Client " + client.getName());

        GeneratorFactory gf = GeneratorFactory.getInstance();
        Document jclient = client.getJonasClientDoc();

        List refs = client.getServiceRefDescs();
        for (Iterator i = refs.iterator(); i.hasNext();) {
            ServiceRefDesc ref = (ServiceRefDesc) i.next();

            // create dd modifier
            Element base = null;
            if (jclient != null) {
                base = jclient.getDocumentElement();
            }
            WsClientDDModifier ddm = new WsClientDDModifier(ref.getServiceRefName(), jclient, base);

            // launch generation
            Generator g = gf.newGenerator(ref, ddm, client);
            g.generate();
            g.compile();
            // add files in web archive
            g.addFiles(client);
        }

        return save(gf.getConfiguration(), "clients" + File.separator + client.getRootFile().getName());
    }
}