/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2004 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: F_JeremieConnector.java,v 1.1 2004/09/28 08:59:48 danesa Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.clients.management;

import java.net.MalformedURLException;
import java.net.URI;
import java.rmi.RemoteException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Properties;
import java.util.StringTokenizer;

import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;
import javax.naming.Context;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.objectweb.carol.util.configuration.CarolConfiguration;
import org.objectweb.carol.util.configuration.CarolCurrentConfiguration;
import org.objectweb.jonas.jtests.util.JTestCase;

/**
* JMX Service test suite
* 
* @author Adriana Danes
* 
*/

public class F_JeremieConnector extends JTestCase {

    /**
     * Constructor
     */
    public F_JeremieConnector(String name) {
        super(name);
    }

    /**
     * init environment
     */
    protected void setUp() {
        super.setUp();
    }

    /**
     * This suite is all test cases
     */
    public static Test suite() {
        return new TestSuite(F_JeremieConnector.class);
    }
    
    public static void main (String args[]) {
        String testtorun = null;
        // Get args
        for (int argn = 0; argn < args.length; argn++) {
            String s_arg = args[argn];
            Integer i_arg;
            if (s_arg.equals("-n")) {
                testtorun = args[++argn];
            }
        }
        if (testtorun == null) {
            junit.textui.TestRunner.run(suite());
        } else {
            junit.textui.TestRunner.run(new F_JeremieConnector(testtorun));
        }
    }

    public void testConnectJmxRemote() throws Exception {
    	// This is a well known MBeans which should be registered in the MBean Server
        // Prepare ObjectName for J2EEServer MBean
    	String sOn = jonasName + ":j2eeType=J2EEServer,name=" + jonasName;
        ObjectName on = null;
        try {
        	on = ObjectName.getInstance(sOn);
        } catch (MalformedObjectNameException e) {
        	// Can't test the connection if no ObjectName
        	fail("Can't create ObjectName for J2EEServer MBean using String: " + sOn);
        }
        // Determine protocols used by Carol and their configuration
        String s = CarolConfiguration.getProtocols();
        StringTokenizer st = new StringTokenizer(s, ",", false);
        Properties jeremieProps = null;
        while (st.hasMoreElements()) {
            CarolCurrentConfiguration carolConfig = CarolCurrentConfiguration.getCurrent();
            String protocol = st.nextToken().trim();
            if (protocol.equals("jeremie")) {
            	jeremieProps = carolConfig.getRMIProperties(protocol);
            }
        }
        if (jeremieProps != null) {
            boolean foundJ2EEServerMBean = true;
        	String sCarolURL = jeremieProps.getProperty(Context.PROVIDER_URL);
            URI carolURL = new URI(sCarolURL);
            int thePort = carolURL.getPort();
        	String port = String.valueOf(thePort + 5); // PATHCH JEREMIE
            String url = "service:jmx:rmi://localhost:" + port + "/jndi/jeremieconnector_" + jonasName;
        	JMXServiceURL connUrl  = new JMXServiceURL(url);
            java.util.Hashtable environment = new java.util.Hashtable();
            environment.put(javax.naming.Context.PROVIDER_URL, "jrmi://localhost:" + thePort);
            environment.put(javax.naming.Context.INITIAL_CONTEXT_FACTORY, "org.objectweb.carol.jndi.spi.JEREMIEContextWrapperFactory");
            environment.put(javax.naming.Context.URL_PKG_PREFIXES, "org.objectweb.jonas.naming");
            
        	JMXConnector connector = JMXConnectorFactory.newJMXConnector(connUrl, environment);
            connector.connect(environment);
            MBeanServerConnection currentServerConnection = connector.getMBeanServerConnection();
            try {
                foundJ2EEServerMBean = currentServerConnection.isRegistered(on);
            } catch (MalformedURLException e) {
            	fail("Can't create JMXServiceURL with string: " + url);
            } catch (java.io.IOException ioe) {
        		fail("Connected, via jeremie, but can't find MBean " + on.toString() + " in the MBeanServer");
            }
    		assertTrue(foundJ2EEServerMBean);
        } else {
        	assertTrue(true);
        }
        
    }
}