/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JBean.java,v 1.1 2004/08/04 15:00:00 durieuxp Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.util;

import javax.ejb.EJBException;
import javax.transaction.Status;
import javax.transaction.UserTransaction;
import javax.transaction.Transaction;
import javax.transaction.TransactionManager;
import org.objectweb.jonas.common.Log;
import org.objectweb.jonas.jtm.TransactionService;
import org.objectweb.jonas.service.ServiceManager;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * This class can be extended in every bean of the jonas test suite.
 * It provide some utilities to look transaction status, ...
 * Most of them are not a clean and standard way to do this, but it
 * is OK for jonas tests.
 */
public class JBean {

    /**
     * @return current Transaction
     */
    public Transaction getCurrentTransaction() {
        try {
            TransactionService ts =
                (TransactionService) ServiceManager.getInstance().getTransactionService();
            TransactionManager tm = ts.getTransactionManager();
            return tm.getTransaction();
        } catch (Exception e) {
            throw new EJBException("Cannot get the current transaction");
        }
    }
            
    /**
     * Utility method that returns true if the thread is associated
     * to a transaction
     */
    public boolean isAssociated() {
        int ret;
        try {
            // Get UserTransaction.
            TransactionService ts =
                (TransactionService) ServiceManager.getInstance().getTransactionService();
            UserTransaction ut = ts.getUserTransaction();
            ret = ut.getStatus();
        } catch (Exception e) {
            System.err.println("isAssociated: "+e);
            return false;
        }
        if (ret == Status.STATUS_UNKNOWN || ret == Status.STATUS_NO_TRANSACTION) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * sleep n millisec.
     */
    public void sleep(int msec) {
        try {
            Thread.sleep(msec);
        } catch (InterruptedException e) {
            System.err.println("sleep interrupted");
        }
    }

}
