/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: F_AxisService.java,v 1.8 2004/09/09 16:17:21 sauthieg Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.clients.service;

import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StringWriter;

import javax.naming.Reference;
import javax.naming.StringRefAddr;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.w3c.dom.Document;

import org.apache.axis.deployment.wsdd.WSDDDeployment;
import org.apache.axis.encoding.SerializationContext;
import org.apache.axis.utils.XMLUtils;

import org.objectweb.jonas_lib.deployment.digester.JDigester;
import org.objectweb.jonas_lib.deployment.rules.JonasServiceRefRuleSet;
import org.objectweb.jonas_lib.deployment.rules.ServiceRefRuleSet;

import org.objectweb.jonas_ws.deployment.api.ServiceRefDesc;

import org.objectweb.jonas.jtests.util.JWebServicesTestCase;
import org.objectweb.jonas.ws.JServiceFactory;
import org.objectweb.jonas.ws.axis.JAxisServiceFactory;

/**
 * test case for axis service instanciation
 */
public class F_AxisService extends JWebServicesTestCase {

    private String resources = null;
    private JServiceFactory jfactory = null;

    public F_AxisService(String name) {
        super(name);
    }
    
    public static Test suite() {
        return new TestSuite(F_AxisService.class);
    }

    public void setUp() throws Exception {
        super.setUp();

        resources = System.getProperty("ws.resources");
        jfactory = new JAxisServiceFactory();
    }

    public void tearDown() throws Exception {
        jfactory = null;
        super.tearDown();
    }


    public void testGetServiceReferenceWithoutConfig() throws Exception {

        // Get a ServiceRef for the test
        ServiceRefDesc sr = createServiceRefDesc("AxisServiceTest-default-std.xml",
                                                 "AxisServiceTest-default-spc.xml");
        
        ClassLoader cl = Thread.currentThread().getContextClassLoader();
        Reference ref = jfactory.getServiceReference(sr, cl);

        String cfg = "<deployment xmlns=\"http://xml.apache.org/axis/wsdd/\" " 
            +                    "xmlns:java=\"http://xml.apache.org/axis/wsdd/providers/java\">"
            +          "<transport name=\"java\" pivot=\"java:org.apache.axis.transport.java.JavaSender\"/>" 
            +          "<transport name=\"http\" pivot=\"java:org.apache.axis.transport.http.HTTPSender\"/>" 
            +          "<transport name=\"local\" pivot=\"java:org.apache.axis.transport.local.LocalSender\"/>"
            +        "</deployment>";


        Reference exp = new Reference("org.objectweb.jonas.jtests.hello.HelloWsServiceLocator",
                                      jfactory.getClass().getName(), null);
        exp.add(new StringRefAddr("client-config.wsdd", cfg));

        exp.add(new StringRefAddr("wsdl-filename", "wsdl/AxisServiceTest-default.wsdl"));
        exp.add(new StringRefAddr("service-qname-namespace", "http://hello.simple"));
        exp.add(new StringRefAddr("service-qname-localpart", "HelloWsService"));

        assertEquals("Reference not created with default config",
                     exp, ref);
    }

    public void testGetServiceReferenceWithConfig() throws Exception {

        // Get a ServiceRef for the test
        ServiceRefDesc sr = createServiceRefDesc("AxisServiceTest-config-std.xml",
                                                 "AxisServiceTest-config-spc.xml");
        
        String cfg = "<deployment xmlns=\"http://xml.apache.org/axis/wsdd/\" " 
            +                    "xmlns:java=\"http://xml.apache.org/axis/wsdd/providers/java\">"
            +          "<service name=\"HelloService\"/>"
            +          "<transport name=\"java\" pivot=\"java:org.apache.axis.transport.java.JavaSender\"/>" 
            +          "<transport name=\"http\" pivot=\"java:org.apache.axis.transport.http.HTTPSender\"/>" 
            +          "<transport name=\"local\" pivot=\"java:org.apache.axis.transport.local.LocalSender\"/>"
            +        "</deployment>";

        ClassLoader cl = Thread.currentThread().getContextClassLoader();
        Reference ref = jfactory.getServiceReference(sr, cl);

        Reference exp = new Reference("org.objectweb.jonas.jtests.hello.HelloWsServiceLocator",
                                      jfactory.getClass().getName(), null);
        exp.add(new StringRefAddr("client-config.wsdd", cfg));

        exp.add(new StringRefAddr("wsdl-filename", "wsdl/AxisServiceTest-default.wsdl"));
        exp.add(new StringRefAddr("service-qname-namespace", "http://hello.simple"));
        exp.add(new StringRefAddr("service-qname-localpart", "HelloWsService"));


        assertEquals("Reference not created with custom config",
                     exp, ref);
        
    }

    public void testGetObjectInstance() throws Exception {
        
        String cfg = "<deployment xmlns=\"http://xml.apache.org/axis/wsdd/\" " 
            +                    "xmlns:java=\"http://xml.apache.org/axis/wsdd/providers/java\">"
            +          "<service name=\"AxisServiceTest\"/>"
            +          "<transport name=\"java\" pivot=\"java:org.apache.axis.transport.java.JavaSender\"/>" 
            +          "<transport name=\"http\" pivot=\"java:org.apache.axis.transport.http.HTTPSender\"/>" 
            +          "<transport name=\"local\" pivot=\"java:org.apache.axis.transport.local.LocalSender\"/>"
            +        "</deployment>";


        Reference ref = new Reference("org.objectweb.jonas.jtests.hello.HelloWsServiceLocator",
                                      jfactory.getClass().getName(), null);
        ref.add(new StringRefAddr("client-config.wsdd", cfg));

        try {
            jfactory.getObjectInstance(ref, null, null, null);
        } catch(Exception e) {
            fail("JAxisServiceFactory.getObjectInstance has failed");
        }

        // cannot success when using Proxies
        /*
        assertTrue("classname not consistent", o instanceof org.objectweb.jonas.jtests.hello.HelloWsServiceLocator);
        
        Service hello = (Service) o;
        WSDDDeployment deploy = ((XMLStringProvider) hello.getEngine().getConfig()).getDeployment();

        assertNotNull("wsdd not instanciated", deploy);

        assertNotNull("service presence", deploy.getService(new QName("AxisServiceTest")));

        assertNotNull("java transport presence", deploy.getTransport(new QName("java")));
        assertNotNull("http transport presence", deploy.getTransport(new QName("http")));
        assertNotNull("local transport presence", deploy.getTransport(new QName("local")));
        */
    }

    public void testSerialization() throws Exception {
        String exp = "<deployment xmlns=\"http://xml.apache.org/axis/wsdd/\" " 
            +                    "xmlns:java=\"http://xml.apache.org/axis/wsdd/providers/java\">"
            +          "<service name=\"HelloService\"/>"
            +          "<transport name=\"java\" pivot=\"java:org.apache.axis.transport.java.JavaSender\"/>" 
            +          "<transport name=\"http\" pivot=\"java:org.apache.axis.transport.http.HTTPSender\"/>" 
            +          "<transport name=\"local\" pivot=\"java:org.apache.axis.transport.local.LocalSender\"/>"
            +        "</deployment>";


        WSDDDeployment base = loadWSDD("org/objectweb/jonas/ws/axis/client-config.wsdd");
        WSDDDeployment deploy = loadWSDD("wsdd/AxisServiceTest-config.wsdd");

        deploy.deployToRegistry(base);

        String ser = serialize(base);

        assertEquals("serialization", exp, ser);
    }
    
    private WSDDDeployment loadWSDD(String res) throws Exception {
        ClassLoader cl = Thread.currentThread().getContextClassLoader();
        InputStream is = cl.getResourceAsStream(res);
        Document deployDoc = XMLUtils.newDocument(is);
        WSDDDeployment deploy = new WSDDDeployment(deployDoc.getDocumentElement());
        return deploy;
    }

    private String serialize(WSDDDeployment d) throws Exception {

        StringWriter sw = new StringWriter();
        SerializationContext ctx = new SerializationContext(sw);
        
        d.writeToContext(ctx);

        return sw.toString();
    }

    private ServiceRefDesc createServiceRefDesc(String srFilename,
                                                String jsrFilename)
        throws Exception {

        // Get Context ClassLoader
        ClassLoader cl = Thread.currentThread().getContextClassLoader();

        ServiceRefTLE srTopLevel= new ServiceRefTLE();
        // Load ServiceRef Object
        //-----------------------------
        // Create Digester
        
        JDigester serviceRefDigester = new JDigester(new ServiceRefRuleSet(""),
                                           false, 
                                           true,
                                           null,
                                           null);
        // Get Reader
        Reader r = new InputStreamReader(cl.getResourceAsStream(srFilename));


        // Parse
        serviceRefDigester.parse(r,
                                 srFilename,
                                 srTopLevel);

        // Load JonasServiceRef Object
        //-----------------------------
        // Create Digester
        JDigester jonasServiceRefDigester = new JDigester(new JonasServiceRefRuleSet(""),
                                                false,
                                                true,
                                                null,
                                                null);
        // Get Reader
        r = new InputStreamReader(cl.getResourceAsStream(jsrFilename));
        // Parse
        jonasServiceRefDigester.parse(r,
                                      jsrFilename,
                                      srTopLevel);

        ServiceRefDesc sr = new ServiceRefDesc(cl,
                                               srTopLevel.getServiceRef(),
                                               srTopLevel.getJonasServiceRef(),
                                               resources);
        return sr;
    }
    
    

    public static void main (String args[]) {
        String testtorun = null;
        // Get args
        for (int argn = 0; argn < args.length; argn++) {
            String s_arg = args[argn];
            if (s_arg.equals("-n")) {
                testtorun = args[++argn];
            }
        }
        if (testtorun == null) {
            junit.textui.TestRunner.run(suite());
        } else {
            junit.textui.TestRunner.run(new F_AxisService(testtorun));
        }
    }

}
